@@ -233,7 +233,7 @@ def _LS_TV_GPU(SMatrix, y, numIterations, isSavingEachIteration, withTumor, lamb
         # Calcul du gradient TV
         theta_2d = theta_flat.reshape(Z, X)
         grad_tv = torch.zeros_like(theta_2d)
-        grad_tv[:-1, :] +=  [1:, :] - theta_2d[:-1, :]  # Dérivée verticale
+        grad_tv[:-1, :] += theta_2d[1:, :] - theta_2d[:-1, :]  # Dérivée verticale
         grad_tv[:, :-1] += theta_2d[:, 1:] - theta_2d[:, :-1]  # Dérivée horizontale
         grad_tv = grad_tv.reshape(Z*X)
 
