@@ -90,17 +90,17 @@ class AlgebraicRecon(Recon):
         if not self.MSE:
             raise ValueError("MSE is empty. Please calculate MSE first.")
         
-        best_idx = np.argmin(self.MSE)
+        best_idx = self.indices[np.argmin(self.MSE)]
         
         print(f"Lowest MSE = {np.min(self.MSE):.4f} at iteration {best_idx+1}")
 
         # Plot MSE curve
         plt.figure(figsize=(7, 5))
-        plt.plot(self.MSE, 'r-', label="MSE curve")
+        plt.plot(self.indices,self.MSE, 'r-', label="MSE curve")
 
         # Add blue dashed lines
         plt.axhline(np.min(self.MSE), color='blue', linestyle='--', label=f"Min MSE = {np.min(self.MSE):.4f}")
-        plt.axvline(best_idx+1, color='blue', linestyle='--', label=f"Iteration = {best_idx+1}")
+        plt.axvline(best_idx, color='blue', linestyle='--', label=f"Iteration = {best_idx+1}")
 
         plt.xlabel("Iteration")
         plt.ylabel("MSE")
@@ -113,11 +113,11 @@ class AlgebraicRecon(Recon):
             now = datetime.now()    
             date_str = now.strftime("%Y_%d_%m_%y")
             if self.optimizer == OptimizerType.MLEM:
-                SavingFolder = os.path.join(self.saveDir, 'results', f'MSE_plot_MLEM{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_MLEM{date_str}.png')
             elif self.optimizer == OptimizerType.LS:
-                SavingFolder = os.path.join(self.saveDir, 'results', f'MSE_plot_LS{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_LS{date_str}.png')
             elif self.optimizer == OptimizerType.LS_TV:
-                SavingFolder = os.path.join(self.saveDir, 'results', f'MSE_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}_{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}_{date_str}.png')
             plt.savefig(SavingFolder, dpi=300)
             print(f"MSE plot saved to {SavingFolder}")
         
@@ -127,7 +127,7 @@ class AlgebraicRecon(Recon):
         if not self.MSE:
             raise ValueError("MSE is empty. Please calculate MSE first.")
 
-        best_idx = np.argmin(self.MSE)
+        best_idx = self.indices[np.argmin(self.MSE)]
         best_recon = self.reconPhantom[best_idx]
 
         # Crée la figure et les axes
@@ -138,7 +138,7 @@ class AlgebraicRecon(Recon):
                             extent=(self.experiment.params.general['Xrange'][0]*1000, self.experiment.params.general['Xrange'][1]*1000,
                                     self.experiment.params.general['Zrange'][1]*1000, self.experiment.params.general['Zrange'][0]*1000),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
-        axs[0].set_title(f"Min MSE Reconstruction\nIter {best_idx+1}, MSE={np.min(self.MSE):.4f}")
+        axs[0].set_title(f"Min MSE Reconstruction\nIter {best_idx}, MSE={np.min(self.MSE):.4f}")
         axs[0].set_xlabel("x (mm)", fontsize=12)
         axs[0].set_ylabel("z (mm)", fontsize=12)
         axs[0].tick_params(axis='both', which='major', labelsize=8)
@@ -194,8 +194,6 @@ class AlgebraicRecon(Recon):
 
         plt.show()
 
-
-
     def show_theta_animation(self, vmin=None, vmax=None, duration=5000, save_path=None):
         """
         Show theta iteration animation (for Jupyter) and optionally save it as a GIF.
@@ -234,7 +232,7 @@ class AlgebraicRecon(Recon):
 
         def update(frame_idx):
             im.set_array(frames[frame_idx])
-            title.set_text(f"Iteration {frame_idx}")
+            title.set_text(f"Iteration {self.indices[frame_idx]}")
             return [im, title]
 
         ani = animation.FuncAnimation(fig, update, frames=num_frames, interval=interval, blit=False)
@@ -257,17 +255,17 @@ class AlgebraicRecon(Recon):
         if not self.SSIM:
             raise ValueError("SSIM is empty. Please calculate SSIM first.")
         
-        best_idx = np.argmax(self.SSIM)
+        best_idx = self.indices[np.argmax(self.SSIM)]
         
         print(f"Highest SSIM = {np.max(self.SSIM):.4f} at iteration {best_idx+1}")
 
         # Plot SSIM curve
         plt.figure(figsize=(7, 5))
-        plt.plot(self.SSIM, 'r-', label="SSIM curve")
+        plt.plot(self.indices,self.SSIM, 'r-', label="SSIM curve")
 
         # Add blue dashed lines
         plt.axhline(np.max(self.SSIM), color='blue', linestyle='--', label=f"Max SSIM = {np.max(self.SSIM):.4f}")
-        plt.axvline(best_idx+1, color='blue', linestyle='--', label=f"Iteration = {best_idx+1}")
+        plt.axvline(best_idx, color='blue', linestyle='--', label=f"Iteration = {best_idx}")
 
         plt.xlabel("Iteration")
         plt.ylabel("SSIM")
@@ -276,10 +274,15 @@ class AlgebraicRecon(Recon):
         plt.grid(True)
         plt.tight_layout()
 
-        if isSaving:
+        if isSaving and self.saveDir is not None:
             now = datetime.now()    
             date_str = now.strftime("%Y_%d_%m_%y")
-            SavingFolder = os.path.join(self.saveDir, 'results', f'SSIM_plot{date_str}.png')
+            if self.optimizer == OptimizerType.MLEM:
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_MLEM{date_str}.png')
+            elif self.optimizer == OptimizerType.LS:
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_LS{date_str}.png')
+            elif self.optimizer == OptimizerType.LS_TV:
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}_{date_str}.png')
             plt.savefig(SavingFolder, dpi=300)
             print(f"SSIM plot saved to {SavingFolder}")
         
@@ -290,7 +293,7 @@ class AlgebraicRecon(Recon):
         if not self.SSIM:
             raise ValueError("SSIM is empty. Please calculate SSIM first.")
 
-        best_idx = np.argmax(self.SSIM)
+        best_idx = self.indices[np.argmax(self.SSIM)]
         best_recon = self.reconPhantom[best_idx]
 
         # ----------------- Plotting -----------------
@@ -303,7 +306,7 @@ class AlgebraicRecon(Recon):
         im0 = axs[0].imshow(best_recon / lambda_max, 
                             extent=(self.experiment.params.general['Xrange'][0], self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
-        axs[0].set_title(f"Max SSIM Reconstruction\nIter {best_idx+1}, SSIM={np.min(self.MSE):.4f}")
+        axs[0].set_title(f"Max SSIM Reconstruction\nIter {best_idx}, SSIM={np.min(self.MSE):.4f}")
         axs[0].set_xlabel("x (mm)")
         axs[0].set_ylabel("z (mm)")
         plt.colorbar(im0, ax=axs[0])
@@ -352,7 +355,7 @@ class AlgebraicRecon(Recon):
         if fin is None:
             fin = len(self.reconPhantom) - 1
 
-        iter_range = range(start, fin+1, step)
+        iter_range = self.indices
 
         crc_values = []
         noise_values = []
@@ -412,7 +415,7 @@ class AlgebraicRecon(Recon):
         if fin is None:
             fin = len(self.reconPhantom) - 1
 
-        iter_list = list(range(start, fin + 1, step))
+        iter_list = self.indices
         nrows = len(iter_list)
         ncols = 3  # Recon, |Recon - GT|, Ground Truth
 
@@ -546,22 +549,22 @@ class AlgebraicRecon(Recon):
             if self.experiment.AOsignal_withTumor is None:
                 raise ValueError("AO signal with tumor is not available. Please generate AO signal with tumor the experiment first in the experiment object.")
             if self.optimizer.value == OptimizerType.MLEM.value:
-                self.reconPhantom = self._MLEM(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
+                self.reconPhantom, self.indices = self._MLEM(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
             elif self.optimizer.value == OptimizerType.LS.value:
-                self.reconPhantom = self._LS(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
+                self.reconPhantom, self.indices = self._LS(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
             elif self.optimizer.value == OptimizerType.LS_TV.value:
-                self.reconPhantom = self._LS_Regularized(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
+                self.reconPhantom, self.indices = self._LS_Regularized(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
             else:
                 raise ValueError(f"Only MLEM and LS are supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
         else:
             if self.experiment.AOsignal_withoutTumor is None:
                 raise ValueError("AO signal without tumor is not available. Please generate AO signal without tumor the experiment first in the experiment object.")
             if self.optimizer.value == OptimizerType.MLEM.value:
-                self.reconLaser = self._MLEM(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
+                self.reconLaser, self.indices = self._MLEM(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
             elif self.optimizer.value == OptimizerType.LS.value:
-                self.reconLaser = self._LS(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
+                self.reconLaser, self.indices = self._LS(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
             elif self.optimizer.value == OptimizerType.LS_TV.value:
-                self.reconLaser = self._LS_Regularized(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
+                self.reconLaser, self.indices = self._LS_Regularized(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
             else:
                 raise ValueError(f"Only MLEM and LS are supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
 
@@ -646,24 +649,25 @@ class AlgebraicRecon(Recon):
         Multi-GPU mode is disabled due to memory fragmentation issues or lack of availability.
         """
         result = None
+        indices = None
         required_memory = calculate_memory_requirement(SMatrix, y)
 
         if self.isGPU:
             if check_gpu_memory(config.select_best_gpu(), required_memory):
-                result = MLEM._MLEM_GPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+                result, indices = MLEM._MLEM_GPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
             else:
                 warnings.warn("Insufficient GPU memory for single GPU MLEM. Falling back to CPU.")
 
         if result is None and self.isMultiCPU:
-            result = MLEM._MLEM_CPU_multi(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+            result, indices = MLEM._MLEM_CPU_multi(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
 
         if result is None:
-            result = MLEM._MLEM_CPU_opti(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+            result, indices = MLEM._MLEM_CPU_opti(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
             if result is None:
                 warnings.warn("Optimized MLEM failed. Falling back to basic CPU MLEM.")
-                result = MLEM._MLEM_CPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+                result, indices = MLEM._MLEM_CPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
 
-        return result
+        return result, indices
     
     def _LS(self, SMatrix, y, withTumor):
         """
@@ -671,31 +675,36 @@ class AlgebraicRecon(Recon):
         Multi-GPU mode is disabled due to memory fragmentation issues or lack of availability.
         """
         result = None
+        indices = None
         required_memory = calculate_memory_requirement(SMatrix, y)
 
         if self.isGPU:
             if check_gpu_memory(config.select_best_gpu(), required_memory):
-                result = LS._LS_GPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+                result, indices = LS._LS_GPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
             else:
                 warnings.warn("Insufficient GPU memory for single GPU LS. Falling back to CPU.")
 
         if result is None and self.isMultiCPU:
-            result = LS._LS_CPU_multi(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+            result, indices = LS._LS_CPU_multi(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
 
         if result is None:
-            result = LS._LS_CPU_opti(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+            result, indices = LS._LS_CPU_opti(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
             if result is None:
                 warnings.warn("Optimized LS failed. Falling back to basic CPU LS.")
-                result = LS._LS_CPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+                result, indices = LS._LS_CPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
 
-        return result
+        return result, indices
 
     def _LS_Regularized(self,SMatrix, y, withTumor):
+        result = None
+        indices = None
         required_memory = calculate_memory_requirement(SMatrix, y)
+
         if check_gpu_memory(config.select_best_gpu(), required_memory):
             if self.lambda_reg is None or self.L_Factor is None:
                 raise ValueError("For LS with TV regularization, both lambda_reg and L_Factor must be specified.")
-            return LS._LS_TV_GPU(SMatrix= SMatrix, y=y,  numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor, lambda_tv=self.lambda_reg, L_Factor=self.L_Factor)
+            result, indices = LS._LS_TV_GPU(SMatrix= SMatrix, y=y,  numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor, lambda_tv=self.lambda_reg, L_Factor=self.L_Factor)
         else:
             warnings.warn("Insufficient GPU memory for single GPU LS with TV regularization. Falling back to CPU.")
-            raise NotImplementedError("LS with TV regularization is not implemented for CPU.")
\ No newline at end of file
+            raise NotImplementedError("LS with TV regularization is not implemented for CPU.")
+        return result, indices
\ No newline at end of file
